/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import java.io.*;
import java.util.*;

/**
   This class is a model for a remote host. It has "probes" for each service
   that is to be monitored on the remote host, as well as status and status
   history. There is one instance of this class for each remote machine MARS
   is watching.
*/

// Revision 0.1: 27 Jan 1999 bht - initial entry
// Revision 0.2: 01 Feb 1999 bht - added visible name support
// Revision 0.3: 07 Feb 1999 bht - fixed javadoc (for use in design doc)
// Revision 0.4: ?? Feb 1999 ben - fixed alert storm in has*changed
// Revision 0.5: 01 Mar 1999 bht - added load & disk limit checking
// Revision 1.1: 12 Mar 1999 bht - attempted to fix a cloning bug
// Revision 1.2.0: 21 Mar 1999 bht - fixed deep cloning bug
// Revision 1.2.2: 25 Mar 1999 bht - changed internal design to 
//                                   facilitate service addition
// Revision 1.3.3: 15 Apr 1999 bht - fixed getDNSName bug
// Revision 1.3.4: 18 Apr 1999 bht - fixed hasSvcChanged logic error
// Revision 1.4.3:  7 May 1999 bht - some hard-coded number fixes, added
//                                   support for IMAP and SSH probing
// Revision 1.4.4: 11 May 1999 bht - fixed the Bowers bug
//  -- THE BOWERS BUG IS: Create a Host. Create some probes. Change the
//     host's DNS name. The DNS name is not updated in the probes.
//     Named for Sean Bowers, who discovered it.
//  -- SOLUTION: Give each probe a reference to its Host. Hostnames are
//     then computed at probe time.
// Revision 1.4.6: 16 May 1999 bht - added per-filesystem free support
//                                   (All we do now is store a hashtable.
//                                   Support for per-fs warnings will come
//                                   soon.)
// Revision 1.4.7: 19 May 1999 bht - changed alertDiskFree to check for
//                                   per-filesystem alerts. alertDiskFree
//                                   now returns a list of filesystems over
//                                   the limit in a string, or NULL for
//                                   no warning.
// Revision 1.4.8:  7 Jun 1999 bht - fixed NPE in alertDiskFree: attempt to
//                                   enumerate null fsInfo.

public class Host implements Serializable, Cloneable {
    
    /** Compile-time debugging switch. Should be false. */
    public static final boolean DEBUG = false;
    
    /** State constant: No status available because probing is disabled. */
    public final static String NA = "";
    /** State constant: An exception occured while probing; no service on 
	the port. Usually means the server process is not running. */
    public final static String FAIL = "down";
    /** State constant: Timeout or unexpected output from server. The server 
	process is running, but is not functioning properly. If this status
	is common, the user should set the probing timeout delay higher */
    public final static String TO = "t/o";
    /** State constant: The service is functioning properly. */
    public final static String OK = "ok";

    // Instance Variables
    String hostname;                       // DNS name of host
    String visiblename;                    // Name used by rest of MARS
    Probe probe[]    = new Probe  [MARS.SVC_COUNT];
    SPOTSQuery spots;                      // SPOTS query probe
    String status[]  = new String [MARS.SVC_COUNT];
    String spot[]    = new String [MARS.SPOT_COUNT];
    String history[] = new String [MARS.SVC_COUNT];
    boolean quellLoadAlert = false;  // Used to determine if load or disk
    boolean quellDiskAlert = false;  // alerts have already been displayed.
    Hashtable fsinfo;

    // Constructors
    
    /**
       Creates a new Host with the specified hostname, and tells the system
       which services to probe.

       @param hostname The DNS name of the host
       @param checkSMTP Whether to check SMTP or not
       @param checkIMAP Whether to check IMAP or not
       @param checkPOP Whether to check POP or not
       @param checkHTTP Whether to check HTTP or not
       @param checkFTP Whether to check FTP or not
       @param checkSSH Whether to check SSH or not
       @param checkSPOTS Whether to query a SPOTS server or not
       @return a new Host with the specified name and services
    */
    public Host (String hostname, 
		 boolean checkSMTP, 
		 boolean checkIMAP,
		 boolean checkPOP,
		 boolean checkHTTP, 
		 boolean checkFTP,
		 boolean checkSSH,
		 boolean checkSPOTS) {

	// Set the hostname
	this.hostname = hostname;
	this.visiblename = hostname;
	
	// Now for each probe type, create a new probe for that type if a
	// the checkTYPE paramater is true.

	setSvcEnable(MARS.SVC_SMTP,checkSMTP);
	setSvcEnable(MARS.SVC_IMAP,checkIMAP);
	setSvcEnable(MARS.SVC_POP,checkPOP);
	setSvcEnable(MARS.SVC_HTTP,checkHTTP);
	setSvcEnable(MARS.SVC_FTP,checkFTP);
	setSvcEnable(MARS.SVC_SSH,checkSSH);
	setSPOTSEnable(checkSPOTS);
    }
    
    /**
       Creates a Host with the specified hostname, but does not enable
       probing of any services on that host.

       @param hostname The DNS name of the host
       @return a new Host with the specified name and no services
    */
    public Host (String hostname) {
	this(hostname,false,false,false,false,false,false,false);
    }

    // Modifiers
    // Probe enablers. These methods set the services that will be probed
    // for this host by creating the probes themselves.

    /**
       Set the host's visible name (that used in the UI and the host table)

       @param visiblename The new visible name of this host
    */
    
    public void setVisibleName (String visiblename) {
	this.visiblename = visiblename;
    }

    /**
       Set the host's DNS name (that used on the network)

       @param hostname The new DNS name of this host
    */
    
    public void setDNSName (String hostname) {
	this.hostname = hostname;
    }

    /**
       Set whether or not to probe a specified numbered service.
       Service names for each number are stored in MARS.SVC[n].

       @param svcNum Number of the service to [en|dis]able.
       @param state whether or not to probe the specified service.
    */
    public void setSvcEnable (int svcNum, boolean state) {
	if (svcNum >= MARS.SVC_COUNT) {
	    throw new Error("Programmer error: svcNum > SVC_COUNT");
	}
	if (state)
	    // Enable probing by creating a new probe for the requested
	    // service
	    switch (svcNum) {
	    case MARS.SVC_SMTP:
		probe[svcNum] = new SMTPProbe(this);
		break;
	    case MARS.SVC_IMAP:
		probe[svcNum] = new IMAPProbe(this);
		break;
	    case MARS.SVC_POP:
		probe[svcNum] = new POPProbe(this);
		break;
	    case MARS.SVC_HTTP:
		probe[svcNum] = new HTTPProbe(this);
		break;
	    case MARS.SVC_FTP:
		probe[svcNum] = new FTPProbe(this);
		break;
	    case MARS.SVC_SSH:
		probe[svcNum] = new SSHProbe(this);
		break;
	    default:
		break;
	    }
	else {
	// otherwise lose our reference to the specified probe and
	// reset the status variable
	    probe[svcNum] = null;
	    setSvcStatus(svcNum,Host.NA);
	}
    }

    /**
       Set whether or not to get system info from SPOTS.
       Creates a new SPOTSQuery on state == true.
       Deletes the current SPOTSQuery on state == false.

       @param state whether to enable SPOTS querying for this host
    */
    public void setSPOTSEnable (boolean state) {
	if (state)
	    // enable probing by creating a new SPOTSQuery
	    spots = new SPOTSQuery(this);
        else {
	    // disable probing by losing our reference to SPOTSQuery
	    spots = null;
	    // make the spots strings NA
	    for (int i=0; i < MARS.SPOT_COUNT; i++)
		setSpotValue(i,null);

	}
    }

    // Status modifiers: used to set the status variables within this class.
    // NOTE: These take any string value. There is no checking to make sure
    // they are one of the Host.XXXX status constants. Be nice.

    /** 
	Set the status of the specified numbered service.
	This _should_ be one of Host.NA, Host.FAIL, Host.TO, or Host.OK.
	
	@param svcNum the service whose status to set
	@param status the status of the service on the remote machine
    */

    public void setSvcStatus (int svcNum, String status) {
	history[svcNum] = this.status[svcNum];
	this.status[svcNum] = status;
    }

    /**
       Set the value of a specified SPOTS-reported system parameter
       This is a string, and should be valid for the numbered field.
       One day, we'll check this here.
    
       @param spotNum the parameter whose value to set
       @param value the value of the specified parameter on the remote machine
    */
    
    public void setSpotValue (int spotNum, String value) {
	spot[spotNum] = value;
    }

    /**
       Set this host's filesystem info hashtable. This is the hash that
       will be retrieved by the UI, and that disk warnings will be checked
       against.

       @param fsinfo the filesystem information hashtable
    */

    public void setFSInfo (Hashtable fsinfo) {
	this.fsinfo = fsinfo;
    }

    // accessor methods

    /**
       Return the status of a specified numbered service.
       This _should_ be one of Host.NA, Host.FAIL, Host.TO, or Host.OK.
    
       @param svcNum the service whose status to get.
       @return the status of the specified service on the remote machine.
    */

    public String getSvcStatus(int svcNum) {
	return status[svcNum];
    }
  
    /**
       Return the value of the specified SPOTS-reported system parameter.
       
       @param spotNum the parameter whose value to get.
       @return the value of the specified parameter.
    */
    public String getSpotValue(int spotNum) {
	return spot[spotNum];
    }

    /**
       Get this host's filesystem info hashtable. This is the hash that
       will be retrieved by the UI, and that disk warnings will be checked
       against.

       @return the filesystem information hashtable
    */

    public Hashtable getFSInfo () {
	return fsinfo;
    }    

    /**
       Return true if there has been a change in the status of the
       specified numbered service since last probe.
    
       @param svcNum the service to check for deltas on
       @return whether the specified service's status has changed
    */

    public boolean hasSvcChanged(int svcNum) {
	/* check for nulls here */
	if (history[svcNum] == null || status[svcNum] == null)
	    return false;

        /* If either history or current is not available, return false */
	/* A lie, but it keeps the system quiet on initial startup */
	if (history[svcNum].equals(Host.NA) || status[svcNum].equals(Host.NA))
	    return false;
	
	/* Otherwise, return true if there is in fact a change */
	if (history[svcNum].equals(status[svcNum]))
	    return false;
	else
	    return true;
    }

    // Probe access (so the ProbeThreads can do thy bidding)
    
    /** 
	Return the Probe for the specified service.

	@param svcNum the service whose probe to return
	@return a Probe for the specified service on this host.
    */
	
    public Probe getSvcProbe (int svcNum) {
	return probe[svcNum];
    }

    /**
       Return the SPOTSQuery probe for this host.

       @return this hosts's SPOTSQuery.
    */

    public SPOTSQuery getSPOTSQuery() {
	return spots;
    }
    
    /**
       Return this Host's visible name (that used by the rest of MARS)
    
       @return this host's visible name
    */

    public String getVisibleName() {
	return visiblename;
    }

    /**
       Return this Host's DNS name (that used on the network)
    
       @return this host's DNS name
    */

    public String getDNSName() {
	return hostname;
    }

    /**
       Determine whether we should alert the user to load average
       overlimit. Uses a quell mechanism to keep from bothering the user
       when unnecessary.
       
       @return true if the user should be notified to a load overlimit
    */

    public boolean alertLoadAvg() {
	/* Start by parsing the load average */
	try {
	    float floatLoad = Float.valueOf(spot[MARS.SPOT_LOAD]).floatValue();
	    if (floatLoad >= MARSConfig.getCurrent().getMaxload()) {
		/* Load average is high. */
		if (quellLoadAlert)
		    return false;           // Quelling load alert, so be quiet
		else {
		    quellLoadAlert = true;
		    return true;            // otherwise alert and remember
		}
	    } else {
		quellLoadAlert = false;
		return false;
	    }
	} catch (Exception ex) {
	    quellLoadAlert = false;
	    return false;
	}
    }

    

    /**
       Determine whether we should alert the user to free disk space
       underlimit. Uses a quell mechanism to keep from bothering the user
       when unnecessary.
       
       @return the name of the filesystem to alert on, or NULL if no alert.
    */

    public String alertDiskFree() {
	String diskFree;
	String fsName;
	String fsWarn = "";
	int percentNdx;
	boolean underLimit = false;

	// Fix NPE: Abort immediately with no warning if no FS info available.
	// (This usually means SPOTS is down)
	if (fsinfo == null) {
	    quellDiskAlert = false;
	    return null;
	}

	/* For each filesystem... */
	Enumeration filesystems = fsinfo.keys();
	while (filesystems.hasMoreElements()) {
	    fsName = (String) filesystems.nextElement();
	    diskFree = (String) fsinfo.get(fsName);

	    /* Start by parsing the disk free space */
	    percentNdx = diskFree.indexOf("%");
	    if (percentNdx >= 0)
		diskFree = diskFree.substring(0,percentNdx);
	    else
		/* Don't bother if we can't find the percent sign */
		continue;
	    
	    try {
		/* Get a numeric representation of the free disk space */
		int intDisk = Integer.parseInt(diskFree);
		/* Compare it with the threshold */
		if (intDisk <= MARSConfig.getCurrent().getMindisk()) {
		    underLimit = true;  /* We're under limit. Alert! */
		    fsWarn += (fsName + " "); 
		}
	    } catch (Exception ex) { continue; }
	}

	/* Under limit and no notification yet. Set quell and return true. */
	if (underLimit && !quellDiskAlert) {
	    quellDiskAlert = true;
	    return fsWarn;
	}
	
	/* We're OK, but we're still quelling. Reset quell and return false */
	if (!underLimit && quellDiskAlert) {
	    quellDiskAlert = false;
	    return null;
	}

	/* Otherwise, we're either OK, or we're quelling. Just return false. */
	return null;
    }

    /**
       Return a string representation of this host.
       This shows user configurable stuff only; NOT status, etc.

       @return a string representing this Host.
    */

    public String toString() {
	return getVisibleName();
    }

    /** 
	Return a clone of this Host
	@return a clone of this host
     */
    public Object clone() {
  	try {
  	    return super.clone();
  	} catch (CloneNotSupportedException ex) {
  	    throw new InternalError();
 	}
    }
}










