/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import java.util.*;

/** 
    There is one ProbeThread for each Host. ProbeThread's job is to
    continually check the status of services on a remote host (as specified
    in the Host class) and update the status of those services in the Host.

    All ProbeThreads are members of a single ThreadGroup, so they can all be
    suspended and resumed at once. ProbeThread's ThreadGroup is accessible by
    the getProbeThreadGroup() static method.
*/

// Revision 0.1: 01 Feb 1999 bht - initial entry
// Revision 0.3: 07 Feb 1999 bht - fixed javadoc (for use in design doc)
// Revision 0.4: 17 Feb 1999 bht - switched to using Vectors for storing
//                                 all thread groups. Added support for
//                                 MARSConfig autowait.
// Revision 0.5: 01 Mar 1999 bht - added support for bounds checking on 
//                               - load avg and free disk
// Revision 1.1: 12 Mar 1999 bht - added support for status bar
// Revision 1.2.1: 21 Mar 1999 bht - changed alert semantics to be less
//                                   annoying.
// Revision 1.2.2: 25 Mar 1999 bht - Structural change. Removed the UI thread
//                                   and cleaned out a bunch of old cruft
//                                   (again, due to a CM problem), and 
//                                   changed the way the system handles 
//                                   multiple services.
// Revision 1.4.6: 16 May 1999 bht - Added per-filesystem info support
//                                   This just shovels info from SPOTSQuery
//                                   into Host.
// Revision 1.4.7: 19 May 1999 bht - changed disk free alert code to accept
//                                   filesystem list instead of a boolean.

public class ProbeThread extends Thread {
    
    // Class variables and initialization
    /** Compile-time debugging switch. Should be false. */
    public static final boolean DEBUG = false;
    
    // Create a Vector for ProbeThreads
    static Vector threadVec = new Vector();
    
    // Instance variable
    Host host;                 // the host we manage probing for.

    /** 
	Create a ProbeThread to manage probing a specified Host.

	@param host the host this ProbeThread will probe.
	@return a ProbeThread for the specified host
    */

    public ProbeThread(Host host) {
	// Add ourselves to the probe thread vector
	threadVec.addElement(this);
	// Set our host to the specified host
	this.host = host;
    }

    /**
       Stop this thread and remove it from the thread vector.
    */
    public void remove() {
	threadVec.removeElement(this);
	this.destroy();
    }
    
    /**
       Start all ProbeThreads
    */
    public static void startAll() {
	// get all probe threads
	Enumeration probeThreads = threadVec.elements();
	ProbeThread thisThread;  // temp storage for each thread

	while (probeThreads.hasMoreElements()) {
	    thisThread = (ProbeThread)(probeThreads.nextElement());
	    thisThread.start();
	}
    }


    /**
       Stop all ProbeThreads
    */
    public static void stopAll() {
	// get all probe threads
	Enumeration probeThreads = threadVec.elements();
	ProbeThread thisThread;  // temp storage for each thread

	while (probeThreads.hasMoreElements()) {
	    thisThread = (ProbeThread)(probeThreads.nextElement());
	    // stop each thread by destroying it
	    // anything less could keep GC from catching threads that
	    // should have died.
	    thisThread.stop();
	}
    }

    /**
       Clear the probe thread vector, so it can be rebuilt.
       This is used during reconfiguration.
    */

    public static void clearAll() {
	threadVec.removeAllElements();
    }

    
    /**
       Probe the specified host forever, updating information in our Host.
    */

    public void run() {
	Probe thisProbe;
	SPOTSQuery spots;
	int i;

	// repeat until the cows come home
	while (true) {
	    for (i=0; i < MARS.SVC_COUNT; i++) {  // for each service
		thisProbe = host.getSvcProbe(i); // get this svc's probe
		if (thisProbe != null) {         // if we are to probe this
 		    MARSMainUI.getCurrent().setStatus(   // set status bar
                                            host+": Checking "+MARS.SVC[i]);
		    try { // try using the probe...
			if (thisProbe.getStatus())        // if probe succeeds
			    host.setSvcStatus(i,Host.OK); // set status OK
			else                              // otherwise
			    host.setSvcStatus(i,Host.TO); // set t/out status
		    } catch (Exception ex) {              // exception means
			host.setSvcStatus(i,Host.FAIL);   // service failure
		    }
		}
	    }

	    // Get system information from SPOTS
	    spots = host.getSPOTSQuery();
	    if (spots != null) {
		MARSMainUI.getCurrent().setStatus(host+": Querying SPOTS");
		try {
		    if (spots.getInfo()) {              // send query to spots
			for (i=0; i < MARS.SPOT_COUNT; i++) // for each spot..
			    host.setSpotValue(i,
				 spots.getSpotValue(i));    // shovel to host
			// now shovel filesystem info over
			host.setFSInfo(spots.getFSInfo());
		    } else {                           // otherwise spotsquery
			for (i=0; i < MARS.SPOT_COUNT; i++) // failed, set to
			    host.setSpotValue(i,Host.TO);   // timeout status
		    }
		} catch (Exception ex) {               // on any failure,
		    for (i=0; i < MARS.SPOT_COUNT; i++)  // set each to fail
			host.setSpotValue(i,Host.FAIL);  // status.
		}

		// Check SPOTS info for over-limit load or disk space
		if (host.alertLoadAvg())
		    MARSMainUI.DisplayBoundsAlert("Load Average",
					 host.getVisibleName(),
					 host.getSpotValue(MARS.SPOT_LOAD),
					 MARSMainUI.getCurrent());
		String fsList = host.alertDiskFree();
		if (fsList != null)
		    MARSMainUI.DisplayDiskAlert(host.getVisibleName(),
						fsList,
						MARSMainUI.getCurrent());
	    }
	    
            // Check the host for changes
	    for (i=0; i < MARS.SVC_COUNT; i++) {   // for each service
		if (host.hasSvcChanged(i) &&       // if the status changed
		    host.getSvcStatus(i).equals(Host.FAIL)) // and is bad
		    // Display a window telling the user which service is
		    // down.
		    MARSMainUI.DisplayDownAlert(MARS.SVC[i],
						host.getVisibleName(),
						MARSMainUI.getCurrent());
	    }
		
	    // Update the table
	    MARSTableModel.updateCurrent();
	    
	    // Check my interrupted flag. Return if necessary.
	    if (isInterrupted()) {
		if (DEBUG)
		 System.out.println
		  ("mars.ProbeThread.run(): "+host+" interrupted late.");
		return;
	    }
	    // Wait for the probe loop time (default = 30 s).
	    MARSMainUI.getCurrent().setStatus(host+": waiting...");
	    try {
		this.sleep(MARSConfig.getCurrent().getAutowait());
	    } catch (Exception ex) {
		// exit on interrupts
		if (DEBUG)
		    System.out.println
			("mars.ProbeThread.run(): "+host+" interrupted.");
		return;
	    }
	}
    }
}






