/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)

  This software (and its source code) are freely usable, modifiable,
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html.

  This software is released without any warranty whatsoever.

  Note this library software is NOT distributed under the GNU LGPL; if you 
  link an application with this software, it is ILLEGAL to distribute it
  under any terms incompatible with the GNU GPL.
*/

package org.altara.netlib;
import org.altara.javalib.Queue;
import java.io.*;
import java.net.*;


/*
  This implementation of the netlib Client interface provides some support
  methods for implementing protocol-based Internet clients (such as the
  included SMTPMessage). Implement such clients by subclassing AbstractClient.
 */

public abstract class AbstractClient implements Client {

    // instance vars
    protected ClientEngine engine;
    protected Queue recvQueue;

    /** Amount of time (in ms) to wait between queue polls in waitFor() */
    public final static long TICK = 10;

    /**
       Create a new AbstractClient connected to a specified host and port.
       
       @param host hostname of server to connect to
       @param port port number to connect to on server
       @return a new ProtocolClient
    */

    public AbstractClient(String host, int port) {

	// Create the Queues to store incoming requests and data.
	recvQueue = new Queue();
      
	// Create a new engine
	engine = new ClientEngine (this, host, port);
    }

    /**
       Recieve data from our ClientEngine.

       @param data the string recieved from the client engine
    */

    public void receive(String data) {
	recvQueue.enqueue(data);         // throw recieved data on the queue.
    }

    /**
       Wait until a string containing a specified string is recieved.

       @param waitString the string to wait for.
       @param timeout number of seconds to wait before returning false.
       @return true if expected data recieved, false otherwise.
    */

    public boolean waitFor(String waitString, int timeout) {

	timeout *= (1000/TICK);            // set up the timeout counter
	boolean found = false;             // found what we're looking for?  
	String cmpString;                  // string to compare to
 
    WAITLOOP:
	while (!found) {                   // loop til we find it...
	    while (recvQueue.isEmpty())    // wait on empty queue.
		try {
		    Thread.currentThread().sleep(TICK);
		    if (timeout-- <= 0)    // decrement timeout counter
			break WAITLOOP;    // breakout if necessary.
		} catch (Exception ex) { break WAITLOOP; }
	    
	    // Dequeue the comparison string
	    cmpString = (String)recvQueue.dequeue();

	    // Make sure it's not null
	    if (cmpString == null)
		continue;

	    // Compare the string with what we're looking for.
	    if (cmpString.indexOf(waitString) >= 0)
		found = true;              // found it!
	}

	return found;
    }

    /**
       Send a string to the server. 
       
       @param data string to send.
    */

    public void send(String data) 
	throws IOException {
	    engine.send(data);
    }

    /**
       Open the connection to the server. 
    */

    public void connect() 
	throws UnknownHostException, IOException {
	engine.connect();
    }
 
    /**
       Close the connection to the server. 
    */

    public void close() { 
	engine.close();
    }
 
    /**
       Handle socket died (null)
    */
    
    public void socketDied() {
    };
}
	    
	    









